import totalReducer from '../common/totalReducer'
import { IRerturnObject } from '../common/typingHelper'
import handleOldGwps from './handleOldGwps'
import { CartWithActions } from '@shopify/hydrogen'

const handleGwp = (cart: CartWithActions, gwpSettings: any) => {
  let returnObject: IRerturnObject = {
    itemsToAdd: [],
    itemsToRemove: handleOldGwps(
      cart,
      gwpSettings.map((s) => s.gift.data.handle)
    ),
  }

  gwpSettings.forEach((settings: any) => {
    const giftHandle = settings.gift.data.handle
    const hasGift =
      cart.lines.filter((i) => i.merchandise.product.handle === giftHandle)
        .length > 0

    const achievedThreshold =
      cart.cost?.subtotalAmount?.amount! >= settings.threshold

    if (!settings.enableGwp || !achievedThreshold) {
      if (hasGift) returnObject.itemsToRemove.push(giftHandle)
      return
    }

    const cleanCartLines = cart.lines.filter((line) => {
      const isCleanProduct = settings.ignoreList
      settings.ignoreList?.filter(
        (i: any) => i.productToIgnore === line.merchandise.product
      ).length === 0

      if (isCleanProduct) return line
    })

    const cleanCartTotal =
      cleanCartLines.length > 0
        ? cleanCartLines?.reduce(totalReducer).cost?.totalAmount.amount
        : cart.cost?.subtotalAmount.amount

    if (cleanCartTotal! < settings.threshold) {
      if (hasGift) returnObject.itemsToRemove.push(giftHandle)
      return
    }

    const manuallyRemoved = window.sessionStorage
      .getItem('manualRemovedGwps')
      ?.includes(giftHandle)

    if (hasGift || manuallyRemoved) return

    returnObject.itemsToAdd.push(settings.gift.data.variants[0].id)
  })

  return returnObject
}

export default handleGwp
